interface NotiProviderSenderResponse {
    id: string;
}
type NotiProviderSender<NotiParams> = (params: NotiParams) => Promise<NotiProviderSenderResponse>;
interface NotiProvider<NotiParams> {
    id: string;
    send: NotiProviderSender<NotiParams>;
}

declare function fallbackStrategy<T>(channelId: string): NotiStrategicSender<T>;

declare function randomStrategy<T>(channelId: string): NotiStrategicSender<T>;

type NotiStrategy<NotiParams> = (channelId: string) => NotiStrategicSender<NotiParams>;
interface NotiStrategicSenderResponse extends NotiProviderSenderResponse {
    providerId: string;
}
type NotiStrategicSender<NotiParams> = (params: NotiParams, providers: Array<NotiProvider<NotiParams>>) => Promise<NotiStrategicSenderResponse>;

type NotiChannelConfig<NotiParams> = {
    id: string;
    providers: Array<NotiProvider<NotiParams>> | ((params: NotiParams) => Promise<Array<NotiProvider<NotiParams>>>);
    strategy: NotiStrategy<NotiParams>;
};
interface NotiChannelSenderResponse extends NotiStrategicSenderResponse {
    channelId: string;
}
type NotiChannelSender<NotiParams> = (params: NotiParams) => Promise<NotiChannelSenderResponse>;
interface NotiChannel<NotiParams> {
    id: string;
    send: NotiChannelSender<NotiParams>;
}
declare function NotiChannel<NotiParams>(config: NotiChannelConfig<NotiParams>): NotiChannel<NotiParams>;

type NotiErrorMetadata<M extends Record<string, boolean | number | string>> = Partial<M & Record<string, boolean | number | string>>;
type NotiErrorOptions<M extends Record<string, boolean | number | string>> = ErrorOptions & {
    errors?: Error[];
    metadata?: NotiErrorMetadata<M>;
};
declare class NotiError<C extends string, M extends Record<string, boolean | number | string>> extends Error {
    code: C;
    metadata: NotiErrorMetadata<M>;
    errors?: Error[];
    constructor(code: C, message: string, options?: NotiErrorOptions<M>);
}
declare class NotiChannelError extends NotiError<'channel_send_failure', {
    channelId: string;
}> {
    static create: (err: unknown, code: "channel_send_failure", options: NotiErrorOptions<{
        channelId: string;
    }> | undefined) => NotiError<any, any>;
}
declare class NotiProviderError extends NotiError<'provider_send_failure', {
    providerId: string;
}> {
    static create: (err: unknown, code: "provider_send_failure", options: NotiErrorOptions<{
        providerId: string;
    }> | undefined) => NotiError<any, any>;
}
declare class NotiTemplateError extends NotiError<'template_not_found' | 'template_renderer_not_found' | 'template_render_failure', {
    templateId: string;
    rendererId?: string;
}> {
    static create: (err: unknown, code: "template_not_found" | "template_renderer_not_found" | "template_render_failure", options: NotiErrorOptions<{
        templateId: string;
        rendererId?: string;
    }> | undefined) => NotiError<any, any>;
}

type NotiTemplateData = Record<string, any>;
type NotiTemplateRendererOptions = Record<string, any>;
type NotiTemplateRenderer = (templatePath: string, data?: NotiTemplateData, options?: NotiTemplateRendererOptions) => Promise<string>;
declare const renderRaw: NotiTemplateRenderer;

type NotiTemplateIdentifier = {
    channel: string;
    topic: string;
    param: string;
};
type NotiTemplateMap = Map<string, string>;
declare function getTemplateMapKey({ channel, topic, param, }: NotiTemplateIdentifier): string;
declare function getTemplateMap(dirPath: string, dirLevel?: number, channel?: string, topic?: string): NotiTemplateMap;

type NotiTemplateConfig = {
    path: string;
    renderer: {
        [templateType: string]: NotiTemplateRenderer;
    };
    data?: Record<string, any>;
    options?: Record<string, any>;
};

type NotiTemplate = {
    templateMap: NotiTemplateMap;
    render: (indentifier: NotiTemplateIdentifier, data?: NotiTemplateData, options?: NotiTemplateRendererOptions) => ReturnType<NotiTemplateRenderer>;
};
declare function NotiTemplate(config: NotiTemplateConfig): NotiTemplate;

export { NotiChannel, type NotiChannelConfig, NotiChannelError, type NotiChannelSender, type NotiChannelSenderResponse, NotiError, type NotiProvider, NotiProviderError, type NotiProviderSender, type NotiProviderSenderResponse, type NotiStrategicSender, type NotiStrategicSenderResponse, type NotiStrategy, NotiTemplate, type NotiTemplateConfig, type NotiTemplateData, NotiTemplateError, type NotiTemplateIdentifier, type NotiTemplateMap, type NotiTemplateRenderer, type NotiTemplateRendererOptions, fallbackStrategy, getTemplateMap, getTemplateMapKey, randomStrategy, renderRaw };
